/********************************************************************
* Copyright Richard Jones June 2002
This file is part of the rspell library.

    The rspell library is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "links.h"

/*Global stuff for debug purposes only*/
#ifndef NDEBUG
long num_collisions=0;
#endif

/*
* Inserts a new node at the beginning of the list, and returns 
* a pointer to the new node
*/
LINK_NODE *add_node(LINK_NODE *node_to_add_to,void *word)
{
LINK_NODE *new_node, *retval;
	
	/*
	* Create a new node and set it's initial values if it works
	*/
	new_node = (LINK_NODE *) malloc(sizeof(LINK_NODE));
	
	if (new_node != NULL)
	{
		new_node->next = NULL;
		new_node->word = word;
		
		/*Set the return value to the new node*/
		retval = new_node;
		
		/*Place the previous node after the new one if required*/
		if (NULL != node_to_add_to)
		{
			new_node->next = node_to_add_to;
			#ifndef NDEBUG
			num_collisions++;
			#endif
		}
	}
	else
	{
		retval = NULL;
	}		

return retval;
}

/*
* Count how many nodes there are in the list from the pointer given
*/
int nodes_in_list(LINK_NODE *lptr)
{
int count=0;

	while (lptr != NULL)
	{
		count++;
		lptr = lptr->next;
	}
	
	return count;		
}

/*
* Remove nodes from the one given to the end.
* Return the number of nodes removed.
* Warning: If you don't give the first node, your previous
* node will point to non-valid memory, giving garbage
*/
int remove_list(LINK_NODE *lptr)
{
int count =0;
LINK_NODE *next_lptr;

	while (lptr != NULL)
	{
		count++;
		next_lptr = lptr->next;
		free (lptr);
		lptr = next_lptr;
	}
	
	return count;
}

/*
* If the memory for the words was malloced, this 
* function will free the memory
* WARNING- you MUST be certain that every word in the list
* was created by allocating memory, or the function will
* have unexpected side effects
*/
void remove_words_from_list(LINK_NODE *top_link)
{
LINK_NODE *lptr;

	lptr=top_link;
	while (lptr != NULL)
	{
		free(lptr->word);
		lptr->word = NULL;
		lptr = lptr->next;
	}

}
