/********************************************************************
* Copyright Richard Jones June 2002
This file is part of the rspell library.

    The rspell library is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*********************************************************************/
/*This file contains various functions used in the spell checker*/

#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <errno.h>
#include "utilities.h"


/*
* Return the size of the file name given, or exit
*/
size_t get_file_size(char *filename)
{
struct stat filestats;	/*file stats*/

	if (!stat(filename,&filestats))
	{
		return filestats.st_size;
	}
	else
	{
		show_error("File stat failure",errno);
		return -1;;
	}

}

/*
* Fill the array with words from the file given
* malloc space for the pointer
* Return 0 on failure
*/
int fill_array(char *filename, char **ptr)
{
FILE *infile;
size_t filesize;
size_t count;
int retval;

	if ((filesize=get_file_size(filename)) < 0)
	{
		retval = FAIL;
	}
	else
	{	
		*ptr=(char *) malloc(filesize * sizeof(char));
		if (ptr != NULL)
		{
			infile=fopen(filename,"r");

			if (NULL == infile)
			{
				show_error("Unable to open file",errno);
				retval = FAIL;
			}
			else
			{
				count = fread (*ptr, 1, filesize, infile);
				assert (count==filesize);
				if (count == filesize)
				{
					retval = 1;
				}
				else
				{
					retval = FAIL;
				}
			}
		}
		else
		{
			show_error ("Malloc Failure",errno);
			retval = FAIL;
		}
	}

return retval;	
}

/*
* Print out information.
* Change this if you want GUI info instead of console
*/
void show_info( char *title, char *info)
{
	/*print the title and info to stdout*/
	
	printf("%s:\n\t%s\n", title, info);
}

/*
* Print out errors.
* Change this if you want GUI info instead of console
* 
*/
void show_error(char *description, int err)
{
	/*Use description and err to print useful error info*/
	if (err != 0)
	{
		fprintf(stderr, "ERROR:%s:\t%s\n", description, strerror(err));
	}
	else
	{
		fprintf(stderr, "ERROR:%s\n", description);
	}
}
